<?php
define('URL_BASE', 'http://codemirror.net/');
#define('URL_BASE', 'https://raw.githubusercontent.com/codemirror/CodeMirror/5.12.0/');

define('STYLE_SHEET', 'style.css');
define('CSS_FILENAME', dirname(__FILE__).'/'.STYLE_SHEET);
define('UGLIFY_HOST', 'http://marijnhaverbeke.nl/uglifyjs');
define('CM_LIB', 'codemirror.min.js');

$stylesheets = array('lib/codemirror', 'addon/display/fullscreen', 'addon/fold/foldgutter', 'addon/dialog/dialog', 'addon/hint/show-hint');
$addons = array(
	'edit/matchbrackets', 'edit/matchtags', 'edit/closebrackets', 'edit/closetag',
	'fold/foldcode', 'fold/foldgutter', 'fold/xml-fold', 'fold/comment-fold', 'fold/markdown-fold',
	'dialog/dialog', 'selection/active-line', 'search/search', 'search/searchcursor',
	'hint/show-hint', 'hint/css-hint', 'hint/html-hint', 'hint/javascript-hint', 'hint/xml-hint'
	);
$modes = array('htmlmixed', 'xml', 'javascript', 'css', 'clike', 'php', 'markdown');

$green = '[32m';
$yellow = '[33m';

function colorMsg($color, $msg) {
	echo chr(27).$color.$msg.chr(27)."[0m]";
}

function print_stylesheets($libRoot) {
	global $stylesheets;

	echo "<!-- print_stylesheets -->\n";
	# style sheets
	if (! is_readable(CSS_FILENAME)) {
		# crée le contenu du fichier style.css
	    $today = date('c');
		$content = "/* ***** Generated by index.php on ${today} ****** */\n\n";
		foreach($stylesheets as $filename) {
			$content .= "/* ***** $libRoot.$filename ******** */\n";
			$content .= file_get_contents($libRoot.$filename)."\n";
		}
		$content .= <<< CONTENT
/* *************** extras **************** */
.CodeMirror {font-size: 10pt; background-color: #f8f0e2;}
.CodeMirror-dialog {background-color: #d5e7f2;}
CONTENT;

		if ($res = file_put_contents(CSS_FILENAME, $content))
			echo <<< STYLE_SHORT
		<link type="text/stylesheet" rel="stylesheet" href="style.css" />

STYLE_SHORT;
		else
			# échec en écriture du fichier style.css
			echo <<< STYLE_LONG
			<style>
	${content}
			</style>

STYLE_LONG;
	} else
		echo '<link rel="stylesheet" type="text/css" href="'.STYLE_SHEET.'" />'."\n";

STYLE;
}

function getFilesList() {
	global $addons, $modes;

	$filesList = ['lib/codemirror'];
	foreach($addons as $addon) {
		$filesList[] = "addon/$addon";
	}
	foreach($modes as $mode) {
		$filesList[] = "mode/$mode/$mode";
	}
	return $filesList;
}

function print_JS($libRoot) {
	$filesList = getFilesList();
	foreach($filesList as $file1) {
		echo <<< SCRIPT_JS
	<script type="text/javascript" src="{$libRoot}{$file1}.js"></script>

SCRIPT_JS;
	}
}

function print_form_download() {
	$urlbase = URL_BASE;
?>
	<form id="form-download" method="post" action="<?php echo UGLIFY_HOST; ?>">
		<p>
			Il est aussi possible de télécharger une librairie minifiée réduite à installer dans le même dossier que ci-dessus.
		</p>
		<div>
			<input name="download" value="<?php echo CM_LIB; ?>" />
<?php
	$fileslist = getFilesList();
	foreach($fileslist as $lib) {
		echo <<< LINE
			<input name="code_url" value="{$urlbase}{$lib}.js" />

LINE;
	} ?>
		</div>
		<p><input type="submit" value="Télécharger"/></p>
	</form>
<?php
}

function download_library() {
	global $green;

	$inputs = array('download=codemirror.min.js');
	$filesList = getFilesList();
	$urlbase = URL_BASE;
	foreach($filesList as $f) {
		$inputs[] = "code_url={$urlbase}/{$f}.js";
	}
	$filename = dirname(__FILE__).'/'.CM_LIB;
	$target = fopen($filename, 'w');
	$curl_options = array(
		CURLOPT_HTTPHEADER		=> array('Content-Type: application/x-www-form-urlencoded'),
		CURLOPT_USERAGENT		=> 'Lynx/2.8.4rel.1 libwww-FM/2.14',
		CURLOPT_FOLLOWLOCATION	=> true,
		CURLOPT_COOKIESESSION	=> true,
		CURLOPT_TIMEOUT			=> 60, // secondes
		CURLOPT_NOPROGRESS		=> false,
		CURLOPT_POST			=> true,
		CURLOPT_POSTFIELDS		=> implode('&',$inputs),
		CURLOPT_FILE			=> $target
	);
	$res = curl_init(UGLIFY_HOST);
	curl_setopt_array($res, $curl_options);
	colorMsg($green, "\nStarting download from ".UGLIFY_HOST." ...\n");
	$resp = curl_exec($res);
	curl_close($res);
	fclose($target);
	if ($resp) {
		echo filesize($filename)." octets reçus.\n\n";
	}
}
/* ***************************************************** */
if (! isset($_SERVER['SERVER_NAME'])) {
	// building style.css file if missing
	$short_name = substr(CSS_FILENAME, strlen(dirname(__FILE__))+1);
	if (! is_file(CSS_FILENAME)) {
		colorMsg($green, "\n$short_name is building...\n");
		$content  = '';
		foreach($stylesheets as $sh) {
			$filename = URL_BASE.$sh.'.css';
			$content .= "/* ------ $filename ------- */\n";
			$content .= file_get_contents($filename);
			echo "$filename downloaded\n";
			}
		$content .= <<< CONTENT
	.CodeMirror {font-size: 10pt; background-color: #f8f0e2;}
	.CodeMirror-dialog {background-color: #FFF;}
CONTENT;
		if (strlen($content) > 0) {
			file_put_contents(CSS_FILENAME, $content);
			echo "$short_name is writing.\n";
		}
	} else {
		echo "$short_name founded\n";
	}
	if (! is_file(dirname(__FILE__).'/'.CM_LIB)) {
		download_library();
	} else {
		echo CM_LIB." founded\n";
	}
	exit;
}
/* ***************** HTML page starts here ************* */
?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
	"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html lang="en">
<head>
	<title>Plugin Codemirror / Tinymce</title>
	<meta http-equiv="content-type" content="text/html;charset=utf-8" />
	<style type="text/css">
		.info, #form-download {
			margin: 10%; padding: 10px; background-color: rgb(251, 243, 227);
			border: 1px solid rgb(245, 177, 38); border-radius: 10px;
			text-align: center;
		}
		#form-download div {display: none;}
		#form-download p {text-align: center;}
		pre {margin: 10px 3em; background-color: rgb(41, 50, 68); text-align: initial; color: rgb(53, 218, 49); padding: 5px 10px;}
		code {margin: 0; font-size: 12pt; }
	</style>
<?php
if (empty($_GET['repo'])) {
	if (is_readable(CSS_FILENAME)) { ?>
	<link rel="stylesheet" type="text/css" href="<?php echo STYLE_SHEET; ?>" />
<?php
	}
	if (is_readable(CM_LIB)) { ?>
	<script type="text/javascript" src="<?php echo CM_LIB; ?>"></script>
<?php
	} else {
		$folders = glob('{codemirror*,default}', GLOB_BRACE | GLOB_ONLYDIR ); // sort by alphabetical order
		if (! empty($folders)) {
			$libRoot = $folders[0].'/';
			print_stylesheets($libRoot);
			print_JS($libRoot);
		} else {
			$libRoot = '';
		}
	}
}
?>
	<script type="text/javascript" src="script.js"></script>
	<script type="text/javascript">
		cm_config = {
			// mode: 'htmlmixed',
			mode: 'application/x-httpd-php',
			lineNumbers: true,
			lineWrapping: true,
			matchBrackets: true,
			autoCloseTags: true,
			autoCloseBrackets: true,
			styleActiveLine: true,
			foldGutter: true,
			gutters: ['CodeMirror-linenumbers', 'CodeMirror-foldgutter']
		}
		window.addEventListener('load', start);
	</script>
</head>
<body>
	<p class="info">
		Il est recommandé d'installer le&nbsp;plugin&nbsp;<a href="http://kazimentou.fr/pluxml-plugins2/index.php?plugin=codemirror&download" target="_blank">Codemirror</a> pour <strong>une&nbsp;meilleure&nbsp;expérience&nbsp;utilisateur</strong><br />
		ou de télécharger l'archive de la librairie codemirror sur <a href="http://codemirror.net">codemirror.net</a><br />
		et de la déplier dans le dossier:<br />
		<?php echo dirname(__FILE__); ?>
	</p>
<?php
if (empty($_GET['repo']) and !is_readable(CM_LIB)) {
 // print_form_download();
?><div class="info">Autre méthode, en mode terminal ou console,<br />placez-vous dans le dossier ci-dessus<br />et lancez la commande suivante :
<pre><code>php -f index.php</code></pre>
</div>
<?php
}
?>
</body>
</html>